Package: WikidataR
Type: Package
Title: Read-Write API Client Library for Wikidata
Version: 2.3.3
Date: 2021-11-16
Authors@R: c(
            person("Thomas", "Shafee", role = c("aut", "cre"), email = "T.Shafee@latrobe.edu.au", comment = c(ORCID = "0000-0002-2298-7593")),
            person("Os", "Keyes", role = "aut", comment = c(ORCID = "0000-0001-5196-609X")),
            person("Serena", "Signorelli", role = "aut"),
            person("Alex", "Lum", role = "ctb", comment = c(ORCID = "0000-0002-9295-9053")),
            person("Christian", "Graul", role = "ctb"),
            person("Mikhail", "Popov", role = "ctb", comment = c(ORCID = "0000-0003-0145-8948"))
           )
Description: Read from, interogate, and write to Wikidata <https://www.wikidata.org> -
    the multilingual, interdisciplinary, semantic knowledgebase. Includes functions to:
    read from wikidata (single items, properties, or properties); query wikidata (retrieving
    all items that match a set of criterial via Wikidata SPARQL query service); write to
    Wikidata (adding new items or statements via QuickStatements); and handle and manipulate
    Wikidata objects (as lists and tibbles). Uses the Wikidata and Quickstatements APIs. 
BugReports: https://github.com/TS404/WikidataR/issues
URL: https://github.com/TS404/WikidataR
License: MIT + file LICENSE
Imports: httr, jsonlite, WikipediR, WikidataQueryServiceR, tibble,
        dplyr, stringr, Hmisc, progress, pbapply, stats, readr, crayon,
        utils
Suggests: markdown, testthat, tidyverse, knitr, pageviews
RoxygenNote: 7.1.1
Encoding: UTF-8
Depends: R (>= 3.5.0)
NeedsCompilation: no
Packaged: 2021-11-16 11:48:41 UTC; thoma
Author: Thomas Shafee [aut, cre] (<https://orcid.org/0000-0002-2298-7593>),
  Os Keyes [aut] (<https://orcid.org/0000-0001-5196-609X>),
  Serena Signorelli [aut],
  Alex Lum [ctb] (<https://orcid.org/0000-0002-9295-9053>),
  Christian Graul [ctb],
  Mikhail Popov [ctb] (<https://orcid.org/0000-0003-0145-8948>)
Maintainer: Thomas Shafee <T.Shafee@latrobe.edu.au>
Repository: CRAN
Date/Publication: 2021-11-16 13:00:06 UTC
